{---------------------------------------------------------------------}
{  The following procedures and functions implement operations on the }
{  abstract data type ListOfValues.  This data type can be used to    }
{  represent a list of integers with values between 0 and             }
{  MaximumListValue.  The following procedures and functions are      }
{  provided:                                                          }
{     InitializeList - initializes a list to contain no values        }
{     AddToList - adds a specified value to a list                    }
{     IsInList - tests whether a specified value is in a list         }
{                                                                     }
{  In this implementation of the type ListOfValues, a list is         }
{  represented as a set of integers with values between 0 and 31.     }
{                                                                     }
{  In order to use this abstract data type, the main program must     }
{  include the constant definition                                    }
{     MaximumListValue = 31                                           }
{  and the type definition                                            }
{     ListOfValues = set of 0..MaximumListValue                       }
{---------------------------------------------------------------------}


{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

procedure InitializeList(var List : ListOfValues);

  {-------------------------------------------------------------------}
  {  InitializeList initializes a list so that it is empty (that is,  }
  {  contains no values).                                             }
  {                                                                   }
  {  Input Parameters: None                                           }
  {                                                                   }
  {  Output Parameters:                                               }
  {    List - the initialized list                                    }
  {-------------------------------------------------------------------}

  begin  { InitializeList }

    List := []

  end;  { InitializeList }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

procedure AddToList(var List: ListOfValues; Value : integer);

  {-------------------------------------------------------------------}
  {  AddToList adds a new value to a list.  If the value being added  }
  {  is already in the list, the list remains unchanged.  If the      }
  {  value being added is outside of the allowed range of values for  }
  {  the list, an error message is displayed.                         }
  {                                                                   }
  {  Input Parameters:                                                }
  {    List - the list to be processed                                }
  {    Value - the new value to be added to the list                  }
  {                                                                   }
  {  Output Parameters:                                               }
  {    List - the list with the new value added                       }
  {-------------------------------------------------------------------}

  begin  { AddToList }

    if (Value >= 0) and (Value <= MaximumListValue) then
      List := List + [Value]
    else
      writeln('*** AddToList: value ',Value:1,' is out of range.')

  end;  { AddToList }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

function IsInList(List : ListOfValues; Value : integer) : boolean;

  {-------------------------------------------------------------------}
  {  IsInList tests whether or not a specified value is in a list.    }
  {  If the value is in the list, IsInList returns the Boolean value  }
  {  true; otherwise, it returns the value false.  If the value       }
  {  is already in the list, the list remains unchanged.  If the      }
  {  value being tested is outside of the allowed range of values for }
  {  the list, an error message is displayed.                         }
  {                                                                   }
  {  Input Parameters:                                                }
  {    List - the list to be processed                                }
  {    Value - the value to be sought in the list                     }
  {                                                                   }
  {  Output Parameters: None                                          }
  {                                                                   }
  {  Function value: true if Value is in List; false otherwise        }
  {-------------------------------------------------------------------}

  begin  { IsInList }

    if (Value >= 0) and (Value <= MaximumListValue) then
      begin

        { Value is in range.  set the function value IsInList to true
          if Value is in List, and false otherwise }

        IsInList := (Value in List)

      end  { if Value is in range }
    else
      begin

        { Value is out of range -- display an error message and return
          false as the function value }

        writeln('*** IsInList: value ',Value:1,' is out of range.');
        IsInList := false
      end  { else }

  end;  { IsInList }
