{---------------------------------------------------------------------}
{  The following procedures and functions implement operations on the }
{  abstract data type ListOfValues.  This data type can be used to    }
{  represent a list of integers with values between 0 and             }
{  MaximumListValue.  The following procedures and functions are      }
{  provided:                                                          }
{     InitializeList - initializes a list to contain no values        }
{     AddToList - adds a specified value to a list                    }
{     IsInList - tests whether a specified value is in a list         }
{                                                                     }
{  In this implementation of the type ListOfValues, a list is         }
{  represented as an array of 25 sets of integers.  Each set may      }
{  contain integers between 0 and 31.  However, the values            }
{  represented by the entries in a set depend on the position of the  }
{  set in the array.  The first set represents values from 0 to 31,   }
{  the second represents values from 32 to 63, and so on.  Thus, an   }
{  entry 0 in the first set represents the value 0; an entry 0 in the }
{  second set represents the value 32, and so on.  The largest value  }
{  that can be entered in a list is 799 (represented by an entry 31   }
{  in the 25th set in the array).                                     }
{                                                                     }
{  In order to use this abstract data type, the main program must     }
{  include the constant definitions                                   }
{     MaximumListValue = 799;                                         }
{     SetsInList = 25                                                 }
{  and the type definition                                            }
{     ListOfValues = array [1..SetsInList] of set of 0..31            }
{---------------------------------------------------------------------}


{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

procedure InitializeList(var List : ListOfValues);

  {-------------------------------------------------------------------}
  {  InitializeList initializes a list so that it is empty (that is,  }
  {  contains no values).                                             }
  {                                                                   }
  {  Input Parameters: None                                           }
  {                                                                   }
  {  Output Parameters:                                               }
  {    List - the initialized list                                    }
  {-------------------------------------------------------------------}

  var
    I : integer;           { used as a subscript for the array of sets }

  begin  { InitializeList }

    for I := 1 to SetsInList do
      List[I] := []

  end;  { InitializeList }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

procedure AddToList(var List: ListOfValues; Value : integer);

  {-------------------------------------------------------------------}
  {  AddToList adds a new value to a list.  If the value being added  }
  {  is already in the list, the list remains unchanged.  If the      }
  {  value being added is outside of the allowed range of values for  }
  {  the list, an error message is displayed.                         }
  {                                                                   }
  {  Input Parameters:                                                }
  {    List - the list to be processed                                }
  {    Value - the new value to be added to the list                  }
  {                                                                   }
  {  Output Parameters:                                               }
  {    List - the list with the new value added                       }
  {-------------------------------------------------------------------}

  var
    SetNumber : integer;      { set number used to represent Value }
    SetEntry : integer;       { entry in set SetNumber that represents Value }

  begin  { AddToList }

    if (Value >= 0) and (Value <= MaximumListValue) then

      begin

        { compute the set number and entry that represent Value}

        SetNumber := Value div 32 + 1;
        SetEntry := Value mod 32;

        { make the entry in the appropriate set }

        List[SetNumber] := List[SetNumber] + [SetEntry]

      end  { if value is in range }
    else
      writeln('*** AddToList: value ',Value:1,' is out of range.')

  end;  { AddToList }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

function IsInList(List : ListOfValues; Value : integer) : boolean;

  {-------------------------------------------------------------------}
  {  IsInList tests whether or not a specified value is in a list.    }
  {  If the value is in the list, IsInList returns the Boolean value  }
  {  true; otherwise, it returns the value false.  If the value       }
  {  is already in the list, the list remains unchanged.  If the      }
  {  value being tested is outside of the allowed range of values for }
  {  the list, an error message is displayed.                         }
  {                                                                   }
  {  Input Parameters:                                                }
  {    List - the list to be processed                                }
  {    Value - the value to be sought in the list                     }
  {                                                                   }
  {  Output Parameters: None                                          }
  {                                                                   }
  {  Function value: true if Value is in List; false otherwise        }
  {-------------------------------------------------------------------}

  var
    SetNumber : integer;      { set number used to represent Value }
    SetEntry : integer;       { entry in set SetNumber that represents Value }

  begin  { IsInList }

    if (Value >= 0) and (Value <= MaximumListValue) then
      begin

        { Value is in range.  Compute the set number and entry that
          represent Value }

        SetNumber := Value div 32 + 1;
        SetEntry := Value mod 32;

        { set the function value IsInList to true if Value is in List,
          and false otherwise }

        IsInList := (SetEntry in List[SetNumber])

      end  { if Value is in range }
    else
      begin

        { Value is out of range -- display an error message and return
          false as the function value }

        writeln('*** IsInList: value ',Value:1,' is out of range.');
        IsInList := false
      end  { else }

  end;  { IsInList }
