function RandomNumber : real;

  {-------------------------------------------------------------------}
  {  RandomNumber returns a random real number between 0 and 1.  The  }
  {  random numbers generated are uniformly distributed in this       }
  {  range.  Before calling RandomNumber, the random-number generator }
  {  should be initialized by a call to procedure InitRandom.         }
  {                                                                   }
  {  RandomNumber calls a non-standard Pascal library routine to      }
  {  generate the random numbers required.                            }
  {                                                                   }
  {  Function value: a random real number between 0 and 1             }
  {-------------------------------------------------------------------}

  begin  { RandomNumber }

    { ## Non-standard statements -- Turbo Pascal version }

    RandomNumber := Random

    { End Turbo Pascal version }

    { ## Non-standard statements -- Berkeley Pascal version }
    {
    RandomNumber := random(1.0)
    }
    { End Berkeley Pascal version }

  end;  { RandomNumber }

{*********************************************************************}

procedure InitRandom(SeedValue : integer);

  {-------------------------------------------------------------------}
  {  InitRandom initializes the random-number generator.  The         }
  {  sequence of random numbers generated is determined by the        }
  {  parameter SeedValue, if this value is not zero.  If SeedValue    }
  {  is zero, the random-number generator is initialized with a       }
  {  value derived from the system clock.  This causes a different    }
  {  sequence of random numbers to be generated each time the program }
  {  is executed.  The procedure calls a non-standard Pascal library  }
  {  routine to perform this operation.                               }
  {                                                                   }
  {  Input parameters:                                                }
  {    SeedValue - a value to be used in initializing the random      }
  {           number generator, or zero                               }
  {                                                                   }
  {  Output parameters: none                                          }
  {                                                                   }
  {  Side effect:  The random number generator is initialized,        }
  {                depending upon the value of SeedValue              }
  {-------------------------------------------------------------------}

  var
    Dummy : integer;        { a dummy variable required by the syntax of
                              Berkeley Pascal -- value is never used }

  begin  { InitRandom }

    { ## Non-standard statements -- Turbo Pascal version }

    if SeedValue = 0 then
      Randomize
    else
      RandSeed := SeedValue

    { End Turbo Pascal version }

    { ## Non-standard statements -- Berkeley Pascal version }
    {
    if SeedValue = 0 then
      Dummy := seed(wallclock)
    else
      Dummy := seed(SeedValue)
    }
    { End Berkeley Pascal version }

  end;  { InitRandom }
