program C3E3(input,output);

{--------------------------------------------------------------------}
{  This program simulates the financing of a purchase over a         }
{  eight-month period, with the payments being drawn from a savings  }
{  account.  Interest on savings is paid monthly.  The program       }
{  displays the amount of interest earned, payment made, new savings }
{  balance, interest paid, and new loan balance for each month.  It  }
{  also calculates and displays the amount of cash (in addition to   }
{  the savings account) needed to pay off the loan after each        }
{  four-month period.                                                }
{                                                                    }
{  Input :   none                                                    }
{            (The amount being financed, monthly payment, starting   }
{             balance in savings, and interest rate on savings are   }
{             given by constants in the program.)                    }
{                                                                    }
{  Output :  Listing of interest earned, payment made, new savings   }
{            balance, interest paid, and new loan balance for each   }
{            month from 1 (first month of loan) to 8 (last month    }
{            of loan).  Listing of cash needed to pay off loan after }
{            each four-month period.                                 }
{--------------------------------------------------------------------}

const
  AmountFinanced = 3000;       { amount financed }
  PaymentAmount = 263.05;      { amount of monthly payment }
  StartingBalance = 3000;      { starting balance in savings account }
  SavingsRate = 0.065;         { annual interest rate on savings }
  LoanRate = 0.095;            { annual interest rate on loan }

var
  Month : integer;             { current month number, starting at 1 }
  SavingsBalance : real;       { current savings balance }
  LoanBalance : real;          { current loan balance }
  InterestEarned : real;       { interest earned for current month }
  InterestPaid : real;         { interest paid for current month   }

{********************************************************************}

procedure Initialize;

  {------------------------------------------------------------------}
  {  Initialize is called once at the beginning of the program.      }
  {  It sets the savings balance and loan balance to their starting  }
  {  values.                                                         }
  {------------------------------------------------------------------}

  begin  { Initialize }

    { set starting balances }

    SavingsBalance := StartingBalance;
    LoanBalance := AmountFinanced

  end;   { Initialize }

{********************************************************************}

procedure WriteHeadings;

  {------------------------------------------------------------------}
  {  WriteHeadings is called once at the beginning of the program.   }
  {  It writes the headings for the listing of monthly results.      }
  {------------------------------------------------------------------}

  begin  { WriteHeadings }

    writeln;
    writeln('         INTEREST              SAVINGS    INTEREST    LOAN  ');
    writeln('MONTH     EARNED    PAYMENT    BALANCE      PAID     BALANCE');
    writeln

  end;   { WriteHeadings }

{********************************************************************}

procedure ComputeNewBalance;

  {------------------------------------------------------------------}
  {  ComputeNewBalance is called once for each month of the loan.    }
  {  It computes the amount of interest earned and the amount of     }
  {  interest paid for the current month.  The new savings balance   }
  {  and new loan balance are computed, reflecting the interest      }
  {  earned, interest paid, and loan payment made.  The amount of    }
  {  additional cash needed to pay off the loan is computed.         }
  {------------------------------------------------------------------}

  begin  { ComputeNewBalance }

    {  compute interest earned and interest paid for one month --
        monthly interest is 1/12 of annual interest }

    InterestEarned := SavingsBalance * (SavingsRate / 12);
    InterestPaid := LoanBalance * (LoanRate / 12);

    {  compute new savings and loan balances }

    SavingsBalance := SavingsBalance + InterestEarned - PaymentAmount;
    LoanBalance := LoanBalance + InterestPaid - PaymentAmount;

    { compute cash needed to pay off the loan }

    CashNeeded := LoanBalance - SavingsBalance

  end;   { ComputeNewBalance }

{********************************************************************}

procedure WriteBalance;

  {------------------------------------------------------------------}
  {  WriteBalance is called once for each month of the loan.  It     }
  {  displays the month number, interest earned, payment made,       }
  {  new savings balance, interest paid, and new loan balance.       }
  {------------------------------------------------------------------}

  begin  { WriteBalance }

    writeln(Month:5, InterestEarned:11:2, PaymentAmount:11:2,
            SavingsBalance:11:2, InterestPaid:11:2, LoanBalance:11:2)

  end;   { WriteBalance }

{********************************************************************}

procedure WriteTotals;

  {------------------------------------------------------------------}
  {  WriteTotals is called once at the end of the program.  It       }
  {  displays the total amount of interest earned and the total      }
  {  amount of interest paid for the term of the loan.               }
  {------------------------------------------------------------------}

  begin  { WriteTotals }

    writeln;
    writeln('TOTAL INTEREST PAID   = ', TotalInterestPaid:8:2);
    writeln('TOTAL INTEREST EARNED = ', TotalInterestEarned:8:2);
    writeln

  end;   { WriteTotals }

{*********************************************************************}

procedure CleanUp;

  {-------------------------------------------------------------------}
  {  CleanUp is called once at the end of the program.  It waits for  }
  {  the user to enter a carriage return before exiting the program.  }
  {-------------------------------------------------------------------}

  begin  { CleanUp }

    write('Press return to exit program');
    readln

  end;  { CleanUp }

{********************************************************************}

begin  { main program }

  { initialize variables and write headings }

  Initialize;
  WriteHeadings;

  { compute and write balances for each month of the loan }

  for Month := 1 to 8 do
    begin
      ComputeNewBalance;
      WriteBalance;
      if (Month = 4) or (Month = 8) then
        WriteCashNeeded
    end;

  { exit program }

  CleanUp

end.
