program Loan3(input,output);

{--------------------------------------------------------------------}
{  This program simulates the financing of a purchase, with the      }
{  with the payments being drawn from a savings account.  Interest   }
{  on savings is paid monthly.  The program displays the amount      }
{  of interest earned, payment made, new savings balance, interest   }
{  paid, and new loan balance for each month.  It continues to       }
{  compute and display results until the loan balance falls to zero. }
{  At the end of the monthly listing, it displays the total amount   }
{  of interest earned on savings, and the total amount of interest   }
{  paid (finance charges).                                           }
{                                                                    }
{  Input :   The starting savings balance, interest rate on savings, }
{            amount being financed, interest rate on the loan, and   }
{            amount of the monthly loan payment are entered from the }
{            keyboard                                                }
{                                                                    }
{  Output :  Listing of interest earned, payment made, new savings   }
{            balance, interest paid, and new loan balance for each   }
{            month.  Listing of total interest earned and total      }
{            interest paid over the period of the loan.              }
{                                                                    }
{  Modification history:                                             }
{            This program performs the same monthly computations as  }
{            the previous version (Loan2).  However, the savings     }
{            balance is never allowed to become negative.  The       }
{            monthly listing continues until the loan balance falls  }
{            to zero.  The output listing pauses after each twelve   }
{            months, waiting for the user to enter a carriage return }
{            before continuing.                                      }
{                                                                    }
{            The initial savings and loan balances, the interest     }
{            rates, and the monthly loan payment are entered by the  }
{            user, instead of being defined as constants in the      }
{            program.  The balances, interest rates, and other       }
{            information for the savings account and the loan        }
{            account are gathered together into records for clarity  }
{            and ease of parameter handling.                         }
{--------------------------------------------------------------------}

type

  SavingsAccount = record                      { savings account information }
                     StartingBalance : real;     { starting balance }
                     SavingsRate : real;         { interest rate }
                     SavingsBalance : real;      { current balance }
                     InterestEarned : real;      { interest earned this month }
                     TotalInterestEarned : real  { total interest to date }
                   end;

  LoanAccount = record                         { loan account information }
                  AmountFinanced : real;         { starting loan amount }
                  LoanRate : real;               { interest rate }
                  RegularPayment : real;         { regular payment amount }
                  CurrentPayment : real;         { current payment amount }
                  LoanBalance : real;            { current balance }
                  InterestPaid : real;           { interest paid this month }
                  TotalInterestPaid : real       { total interest to date }
                end;

var
  Savings : SavingsAccount;    { savings account information }
  Loan : LoanAccount;          { loan information }
  Month : integer;             { current month number, starting at 1 }

{**********************************************************************}

procedure Initialize(var Savings : SavingsAccount; var Loan : LoanAccount);

  {------------------------------------------------------------------}
  {  Initialize is called once at the beginning of the program.      }
  {  It prompts the user to enter starting savings and loan balances }
  {  and interest rates, and the amount of the monthly loan payment. }
  {  The values entered are stored in the records that represent the }
  {  savings and loan accounts, and the totals are initialized to    }
  {  zero.                                                           }
  {                                                                  }
  {  Input parameters: None                                          }
  {                                                                  }
  {  Output parameters:                                              }
  {    Savings - savings account information at start of loan        }
  {    Loan - loan account information at start of loan              }
  {------------------------------------------------------------------}

  begin  { Initialize }

    { get starting savings balance }

    write('Starting savings balance     : ');
    readln(Savings.StartingBalance);

    { get savings interest rate, converting percentage to decimal fraction }

    write('Interest rate on savings (%) : ');
    readln(Savings.SavingsRate);
    Savings.SavingsRate := Savings.SavingsRate / 100;

    { get starting loan amount }

    write('Amount financed on loan      : ');
    readln(Loan.AmountFinanced);

    { get loan interest rate, converting percentage to decimal fraction }

    write('Interest rate on loan (%)    : ');
    readln(Loan.LoanRate);
    Loan.LoanRate := Loan.LoanRate / 100;

    { get amount of monthly loan payment }

    write('Monthly payment on loan      : ');
    readln(Loan.RegularPayment);

    { set starting balances for savings account and loan }

    Savings.SavingsBalance := Savings.StartingBalance;
    Loan.LoanBalance := Loan.AmountFinanced;

    { initialize totals to zero }

    Savings.TotalInterestEarned := 0;
    Loan.TotalInterestPaid := 0

  end;   { Initialize }

{********************************************************************}

procedure WriteHeadings;

  {------------------------------------------------------------------}
  {  WriteHeadings is called once at the beginning of the program.   }
  {  It writes the headings for the listing of monthly results.      }
  {------------------------------------------------------------------}

  begin  { WriteHeadings }

    writeln;
    writeln('         INTEREST              SAVINGS    INTEREST    LOAN  ');
    writeln('MONTH     EARNED    PAYMENT    BALANCE      PAID     BALANCE');
    writeln

  end;   { WriteHeadings }

{********************************************************************}

procedure ComputeNewBalance(var Savings : SavingsAccount;
                            var Loan : LoanAccount);

  {------------------------------------------------------------------}
  {  ComputeNewBalance is called once for each month of the loan.    }
  {  It computes the amount of interest earned and the amount of     }
  {  interest paid for the current month.  The new savings balance   }
  {  and new loan balance are computed, reflecting the interest      }
  {  earned, interest paid, and loan payment made.  The interest     }
  {  earned and interest paid are added to the totals.               }
  {                                                                  }
  {  Input parameters:                                               }
  {    Savings - savings account information at start of the         }
  {              current month                                       }
  {    Loan - loan account information at start of the current month }
  {                                                                  }
  {  Output parameters:                                              }
  {    Savings - savings account information at end of current month }
  {    Loan - loan account information at end of the current month   }
  {------------------------------------------------------------------}

  begin  { ComputeNewBalance }

    {  compute interest earned and interest paid for one month --
        monthly interest is 1/12 of annual interest }

    Savings.InterestEarned :=
                     Savings.SavingsBalance * (Savings.SavingsRate / 12);
    Loan.InterestPaid := Loan.LoanBalance * (Loan.LoanRate / 12);

    {  compute amount owed on the loan, including interest }

    Loan.LoanBalance := Loan.LoanBalance + Loan.InterestPaid;

    {  set the current payment equal to the regular payment or the amount
       owed, whichever is smaller }

    if Loan.LoanBalance > Loan.RegularPayment then
      Loan.CurrentPayment := Loan.RegularPayment
    else
      Loan.CurrentPayment := Loan.LoanBalance;

    {  subtract the current payment to find the new loan balance }

    Loan.LoanBalance := Loan.LoanBalance - Loan.CurrentPayment;

    {  compute new savings balance }

    Savings.SavingsBalance := Savings.SavingsBalance
                                + Savings.InterestEarned - Loan.CurrentPayment;

    {  if the savings balance would become negative, set it to zero }

    if Savings.SavingsBalance < 0 then
      Savings.SavingsBalance := 0;

    {  add interest earned and interest paid to totals }

    Savings.TotalInterestEarned := Savings.TotalInterestEarned
                                         + Savings.InterestEarned;
    Loan.TotalInterestPaid := Loan.TotalInterestPaid + Loan.InterestPaid

  end;   { ComputeNewBalance }

{********************************************************************}

procedure WriteBalance(Month : integer; Savings : SavingsAccount;
                       Loan : LoanAccount);

  {------------------------------------------------------------------}
  {  WriteBalance is called once for each month of the loan.  It     }
  {  displays the month number, interest earned, payment made,       }
  {  new savings balance, interest paid, and new loan balance.       }
  {                                                                  }
  {  Input parameters:                                               }
  {    Month - current month number                                  }
  {    Savings - savings account information at end of current month }
  {    Loan - loan account information at end of the current month   }
  {                                                                  }
  {  Output parameters : None                                        }
  {------------------------------------------------------------------}

  begin  { WriteBalance }

    { write savings and loan information for current month }

    writeln(Month:5, Savings.InterestEarned:11:2, Loan.CurrentPayment:11:2,
            Savings.SavingsBalance:11:2, Loan.InterestPaid:11:2,
            Loan.LoanBalance:11:2);

    { if current month is a multiple of 12, wait until the user enters
      a carriage return before continuing }

    if (Month mod 12) = 0 then
      begin
        writeln;
        write('Press return to continue');
        readln
      end

  end;   { WriteBalance }

{********************************************************************}

procedure WriteTotals(Savings : SavingsAccount; Loan : LoanAccount);

  {------------------------------------------------------------------}
  {  WriteTotals is called once at the end of the program.  It       }
  {  displays the total amount of interest earned and the total      }
  {  amount of interest paid for the months covered in the listing.  }
  {                                                                  }
  {  Input parameters:                                               }
  {    Savings - savings account information at end of final month   }
  {    Loan - loan account information at end of final month         }
  {                                                                  }
  {  Output parameters: None                                         }
  {------------------------------------------------------------------}

  begin  { WriteTotals }

    writeln;
    writeln('TOTAL INTEREST PAID   = ', Loan.TotalInterestPaid:8:2);
    writeln('TOTAL INTEREST EARNED = ', Savings.TotalInterestEarned:8:2);
    writeln

  end;   { WriteTotals }

{*********************************************************************}

procedure CleanUp;

  {-------------------------------------------------------------------}
  {  CleanUp is called once at the end of the program.  It waits for  }
  {  the user to enter a carriage return before exiting the program.  }
  {-------------------------------------------------------------------}

  begin  { CleanUp }

    write('Press return to exit program');
    readln

  end;  { CleanUp }

{********************************************************************}

begin  { main program }

  { initialize variables and write headings }

  Initialize(Savings,Loan);
  WriteHeadings;

  { compute and write balances for each month of the loan }

  Month := 0;
  while Loan.LoanBalance > 0 do
    begin
      Month := Month + 1;
      ComputeNewBalance(Savings,Loan);
      WriteBalance(Month,Savings,Loan)
    end;

  { write totals and exit program }

  WriteTotals(Savings,Loan);
  CleanUp

end.
