program Cipher(input,output,KEYS);

{---------------------------------------------------------------------}
{  This program enciphers and deciphers messages using a simple       }
{  substitution cipher.  Messages are entered from the keyboard,      }
{  along with a command code requesting that they be enciphered  or   }
{  deciphered.  The keys used for enciphering and deciphering are     }
{  read from file KEYS.  The result of the requested enciphering or   }
{  deciphering is written back to the terminal.                       }
{                                                                     }
{  The user is prompted for each command to be entered.  The possible }
{  commands are                                                       }
{         E - encipher the following message                          }
{         D - decipher the following message                          }
{         Q - quit execution of the program                           }
{  If the command entered is E or D, the user is then prompted to     }
{  enter the message to be enciphered or deciphered.                  }
{                                                                     }
{  Commands and messages may be entered using any combination of      }
{  upper case and lower case letters.  The output messages produced   }
{  by the program are in upper case for deciphered messages and lower }
{  case for enciphered messages.  Non-alphabetic characters may be    }
{  included in messages, and are reproduced in the enciphered or      }
{  deciphered message without change.                                 }
{                                                                     }
{  The enciphering and deciphering keys are stored in file KEYS in    }
{  two lines.  The first line gives the encipherment for each         }
{  (plaintext) letter 'a' through 'z', in lower case.  The second     }
{  line gives the decipherment of each (ciphertext) letter            }
{  'a' through 'z', in upper case.  The enciphering key should        }
{  contain only lower case characters, and the deciphering key should }
{  contain only upper case characters.  For example, the enciphering  }
{  key might be                                                       }
{                   owbsyfdahnegpqiurtljkmzxcv                        }
{  and the corresponding deciphering key might be                     }
{                   HCYGKFLIOTUSVJAMNQDRPZBXEW                        }
{                                                                     }
{  Input:  Commands and messages as described above, entered from the }
{          terminal.  Enciphering and deciphering keys, read from     }
{          file KEYS.                                                 }
{                                                                     }
{  Output: Enciphered and deciphered messages, displayed on the       }
{          terminal.                                                  }
{                                                                     }
{  This program contains non-standard Pascal statements to support    }
{  Turbo Pascal and Berkeley Pascal versions.  Non-standard Pascal    }
{  statements appear in the following locations:                      }
{    1. in procedures Initialize and CleanUp                          }
{  These non-standard statements are preceded by a comment that       }
{  begins with '## N'.                                                }
{---------------------------------------------------------------------}

const
  MaximumLength = 80;          { maximum length of message to be processed }

type
  Message = array[1..MaximumLength] of char;
  Key = array['a'..'z'] of char;

var
  KEYS : text;                { file containing keys }
  EncipherKey : Key;          { enciphering key }
  DecipherKey : Key;          { deciphering key }
  Command : char;             { command entered from terminal }
  Plaintext : Message;        { message to be enciphered }
  Ciphertext : Message;       { message to be deciphered }
  MessageLength : integer;    { length of message being processed }

{*********************************************************************}

function Lowercase(Letter : char) : char;

  {-------------------------------------------------------------------}
  {  Lowercase is used to convert alphabetic characters to lower      }
  {  case.  If the input parameter is an upper case letter            }
  {  ('A' through 'Z'), the value of the function is the corresponding}
  {  lower case letter ('a' through 'z').  If the input parameter is  }
  {  any other character, the value of the function is equal to the   }
  {  input parameter.                                                 }
  {                                                                   }
  {  Input parameters:                                                }
  {    Letter - character to be converted to lower case               }
  {                                                                   }
  {  Output parameters : None                                         }
  {                                                                   }
  {  Function value : the value of the input parameter Letter,        }
  {                   converted to lower case if it was originally    }
  {                   in the range 'A' through 'Z'                    }
  {-------------------------------------------------------------------}

  var
    ConvertedLetter : char;       { the function value being computed }
    DistancePastA : integer;      { the distance in the alphabet between
                                    the letter being converted and 'A' }
  begin  { Lowercase }

    { if the input parameter is an upper case letter, convert it to
      the corresponding lower case letter.  otherwise, the converted
      letter is set equal to the input parameter. }

    if (Letter >= 'A') and (Letter <= 'Z') then
      begin
        DistancePastA := ord(Letter) - ord('A');
        ConvertedLetter := chr(ord('a') + DistancePastA)
      end  { if }
    else
      ConvertedLetter := Letter;

    { set the function value }

    Lowercase := ConvertedLetter

  end;  { Lowercase }

{*********************************************************************}

function Uppercase(Letter : char) : char;

  {-------------------------------------------------------------------}
  {  Uppercase is used to convert alphabetic characters to upper      }
  {  case.  If the input parameter is a lower case letter             }
  {  ('a' through 'z'), the value of the function is the corresponding}
  {  upper case letter ('A' through 'Z').  If the input parameter is  }
  {  any other character, the value of the function is equal to the   }
  {  input parameter.                                                 }
  {                                                                   }
  {  Input parameters:                                                }
  {    Letter - character to be converted to upper case               }
  {                                                                   }
  {  Output parameters : None                                         }
  {                                                                   }
  {  Function value : the value of the input parameter Letter,        }
  {                   converted to upper case if it was originally    }
  {                   in the range 'a' through 'z'                    }
  {-------------------------------------------------------------------}

  var
    ConvertedLetter : char;       { the function value being computed }
    DistancePastA : integer;      { the distance in the alphabet between
                                    the letter being converted and 'a' }
  begin  { Uppercase }

    { if the input parameter is a lower case letter, convert it to
      the corresponding upper case letter.  otherwise, the converted
      letter is set equal to the input parameter. }

    if (Letter >= 'a') and (Letter <= 'z') then
      begin
        DistancePastA := ord(Letter) - ord('a');
        ConvertedLetter := chr(ord('A') + DistancePastA)
      end  { if }
    else
      ConvertedLetter := Letter;

    { set the function value }

    Uppercase := ConvertedLetter

  end;  { Uppercase }

{*********************************************************************}

procedure Initialize;

  {-------------------------------------------------------------------}
  {  Initialize is called once at the beginning of the program.  It   }
  {  opens the input file KEYS.                                       }
  {-------------------------------------------------------------------}

  begin  { Initialize }

    { ## Non-standard statements -- Turbo Pascal version }

    assign(KEYS,'KEYS');

    { End Turbo Pascal version }

    reset(KEYS);

  end;  { Initialize }

{*********************************************************************}

procedure GetKeys(var EncipherKey,DecipherKey : Key);

  {-------------------------------------------------------------------}
  {  GetKeys is called once.  It reads the file KEYS and stores the   }
  {  enciphering and deciphering keys for later use.  Notice that     }
  {  both enciphering and deciphering keys are stored in arrays       }
  {  with subscripts ranging from 'a' to 'z'.                         }
  {                                                                   }
  {  Input parameters: None                                           }
  {                                                                   }
  {  Output parameters:                                               }
  {    EncipherKey - the enciphering key to be used                   }
  {    DecipherKey - the deciphering key to be used                   }
  {-------------------------------------------------------------------}

  var
    Letter : char;    { used as a subscript for EncipherKey and DecipherKey }

  begin

    { read enciphering and deciphering keys }

    for Letter := 'a' to 'z' do
      read(KEYS,EncipherKey[Letter]);

    readln(KEYS);

    for Letter := 'a' to 'z' do
      read(KEYS,DecipherKey[Letter]);

    readln(KEYS)

  end;  { GetKeys }

{*********************************************************************}

procedure GetCommand(var Command : char);

  {-------------------------------------------------------------------}
  {  GetCommand is called each time a command is to be read from the  }
  {  terminal.  It prompts the user to enter a command, then reads    }
  {  a character from the terminal and converts it to upper case.     }
  {  The converted character is returned in the parameter Command.    }
  {                                                                   }
  {  Input parameters: None                                           }
  {                                                                   }
  {  Output parameters:                                               }
  {    Command - the command character read, converted to upper case  }
  {-------------------------------------------------------------------}

  var
    Letter : char;    { command letter read from the terminal }

  begin  { GetCommand }

    write('Enter command (E,D,Q) : ');
    readln(Letter);
    Command := Uppercase(Letter)

  end;  { GetCommand }

{*********************************************************************}

procedure ReadMessage(var InputMessage : Message; var MessageLength : integer);

  {-------------------------------------------------------------------}
  {  ReadMessage is called whenever the command entered was E or D.   }
  {  It prompts the user to enter a message to be enciphered or       }
  {  deciphered, and then reads this message from the terminal.  The  }
  {  message read is returned in the parameter InputMessage, and its  }
  {  length is returned in parameter MessageLength.  If the message   }
  {  entered is shorter than MaximumLength, the rest of InputMessage  }
  {  is filled with blanks.  If the message is longer than the        }
  {  maximum length allowed, the excess characters are ignored and a  }
  {  warning message is displayed on the terminal.                    }
  {                                                                   }
  {  Input parameters: None                                           }
  {                                                                   }
  {  Output parameters:                                               }
  {    Message - the message read from the terminal                   }
  {    MessageLength - the length of the message read                 }
  {-------------------------------------------------------------------}

  var
    CharacterCount : integer;        { number of characters read so far }
    I : integer;                     { used as a subscript for InputMessage }

  begin

    { prompt user for input }

    writeln('Enter message:');
    CharacterCount := 0;

    { read characters from the terminal until end-of-line is detected,
      or the maximum size message has been read }

    while (not eoln(input)) and (CharacterCount < MaximumLength) do
      begin
        CharacterCount := CharacterCount + 1;
        read(InputMessage[CharacterCount])
      end;

    { fill unused elements in InputMessage with blanks }

    for I := CharacterCount + 1 to MaximumLength do
      InputMessage[I] := ' ';

    { if end-of-line was detected, the message was of a proper length.
      otherwise, the message was too long -- issue a warning to the user. }

    if not eoln(input) then
        writeln('*** Message truncated to maximum length of ',
                MaximumLength:3,' characters.');

    { skip the end-of-line marker and any input remaining on the current line }

    readln;

    { set the message length }

    MessageLength := CharacterCount

  end;  { ReadMessage }

{*********************************************************************}

procedure ApplyCode(Original : Message; MessageLength : integer;
                        KeyToUse : Key; var Transformed : Message);

  {-------------------------------------------------------------------}
  {  ApplyCode is called whenever a message is to be enciphered or    }
  {  deciphered.  The message given in the parameter Original is      }
  {  transformed by applying the (enciphering or deciphering) key     }
  {  given in KeyToUse.  The result is returned in the parameter      }
  {  Transformed.  The length of the message being processed is given }
  {  by MessageLength.                                                }
  {                                                                   }
  {  Input parameters:                                                }
  {    Original - the message to be enciphered or deciphered          }
  {    MessageLength - the length of the message                      }
  {    KeyToUse - the enciphering or deciphering key to be applied    }
  {               to Original                                         }
  {                                                                   }
  {  Output parameters:                                               }
  {    Transformed - the enciphered or deciphered message that        }
  {                  results from applying KeyToUse to Original       }
  {-------------------------------------------------------------------}

  var
    I : integer;          { used as subscript for Original and Transformed }
    Character : char;     { character being enciphered or deciphered }

  begin  { ApplyCode }

    for I := 1 to MessageLength do
      begin

        { convert upper case character to lower case }

        Character := Lowercase(Original[I]);

        { if character is alphabetic, transform it according to the key;
          otherwise, copy the character without change }

        if (Character >= 'a') and (Character <= 'z') then
          Transformed[I] := KeyToUse[Character]
        else
          Transformed[I] := Character
      end  { for }

  end;  { ApplyCode }

{*********************************************************************}

procedure WriteMessage(OutputMessage : Message; MessageLength : integer);

  {-------------------------------------------------------------------}
  {  WriteMessage is called whenever an enciphered or deciphered      }
  {  message is to be written back to the terminal.                   }
  {                                                                   }
  {  Input parameters:                                                }
  {    OutputMessage - the message to be displayed                    }
  {    MessageLength - the length of the message                      }
  {                                                                   }
  {  Output parameters: None                                          }
  {-------------------------------------------------------------------}

  var
    I : integer;          { used as subscript for OutputMessage }

  begin  { WriteMessage }

    for I := 1 to MessageLength do
      write(OutputMessage[I]);
    writeln

  end;  { WriteMessage }

{*********************************************************************}

procedure CleanUp;

  {-------------------------------------------------------------------}
  {  CleanUp is called once at the end of the program.  It closes     }
  {  the file KEYS, if this is required in the Pascal version being   }
  {  used.                                                            }
  {-------------------------------------------------------------------}

  begin  { CleanUp }

    { ## Non-standard statements -- Turbo Pascal version }

    close(KEYS)

    { End Turbo Pascal version }

  end;  { CleanUp }

{*********************************************************************}

begin  { main program }

  Initialize;

  { read enciphering and deciphering keys and get first command }

  GetKeys(EncipherKey,DecipherKey);
  GetCommand(Command);

  { read and process commands until command Q is entered }

  while Command <> 'Q' do
    begin
      if Command = 'E' then
        begin

          { read message, encipher it, and write enciphered version }

          ReadMessage(Plaintext,MessageLength);
          ApplyCode(Plaintext,MessageLength,EncipherKey,Ciphertext);
          WriteMessage(Ciphertext,MessageLength)
        end  { if command was E }
      else if Command = 'D' then
        begin

          { read message, decipher it, and write deciphered version }

          ReadMessage(Ciphertext,MessageLength);
          ApplyCode(Ciphertext,MessageLength,DecipherKey,Plaintext);
          WriteMessage(Plaintext,MessageLength)
        end  { if command was D }
      else

        { command was not a valid choice -- write error message }

        writeln('Commands are E(ncipher), D(ecipher), Q(uit)');

      { get the next command }

      GetCommand(Command)
    end;  { while command not Q }

  { exit program }

  CleanUp

end.
