procedure ReadMessage(var InputMessage : Message; var MessageLength : integer);

  {-------------------------------------------------------------------}
  {  ReadMessage is called whenever the command entered was E or D.   }
  {  It prompts the user to enter a message to be enciphered or       }
  {  deciphered, and then reads this message from the terminal.  The  }
  {  message read is returned in the parameter InputMessage, and its  }
  {  length is returned in parameter MessageLength.  If the message   }
  {  entered is shorter than MaximumLength, the rest of InputMessage  }
  {  is filled with blanks.  If the message is longer than the        }
  {  maximum length allowed, the excess characters are ignored and a  }
  {  warning message is displayed on the terminal.                    }
  {                                                                   }
  {  Input parameters: None                                           }
  {                                                                   }
  {  Output parameters:                                               }
  {    Message - the message read from the terminal                   }
  {    MessageLength - the length of the message read                 }
  {-------------------------------------------------------------------}

  var
    CharacterCount : integer;        { number of characters read so far }
    I : integer;                     { used as a subscript for InputMessage }

  begin

    { prompt user for input }

    writeln('Enter message:');
    CharacterCount := 0;

    { read characters from the terminal until end-of-line is detected,
      or the maximum size message has been read }

    while (not eoln(input)) and (CharacterCount < MaximumLength) do
      begin
        CharacterCount := CharacterCount + 1;
        read(InputMessage[CharacterCount])
      end;

    { fill unused elements in InputMessage with blanks }

    for I := CharacterCount + 1 to MaximumLength do
      InputMessage[I] := ' ';

    { if end-of-line was detected, the message was of a proper length.
      otherwise, the message was too long -- issue a warning to the user. }

    if not eoln(input) then
        writeln('*** Message truncated to maximum length of ',
                MaximumLength:3,' characters.');

    { skip the end-of-line marker and any input remaining on the current line }

    readln;

    { set the message length }

    MessageLength := CharacterCount

  end;  { ReadMessage }
