program MakeKeys(input,output,KEYS);

{---------------------------------------------------------------------}
{  This program creates a file named KEYS that contains enciphering   }
{  and deciphering keys for use by the program Cipher.  The first     }
{  line of the file contains the enciphering key, and the second      }
{  line contains the deciphering key.                                 }
{                                                                     }
{  To generate these keys, the user is first prompted to enter a      }
{  string of letters from the terminal.  An enciphering key is        }
{  generated by taking these letters in the order they were entered   }
{  (with duplicate letters eliminated), followed by the letters       }
{  of the alphabet that do not appear in the string (in reverse       }
{  alphabetical order).  This sequence of letters gives the           }
{  encipherment for each (plaintext) letter 'a' through 'z', in       }
{  lower case.  The case of the letters entered by the user is not    }
{  significant.                                                       }
{                                                                     }
{  The deciphering key gives the decipherment of each (ciphertext)    }
{  letter 'a' through 'z', in upper case.  This key is generated to   }
{  be the inverse of the enciphering key.                             }
{                                                                     }
{  For example, if the user entered the string                        }
{                Communications of the ACM                            }
{  the enciphering key would be                                       }
{                comuniatsfhezyxwvrqplkjgdb                           }
{  and the deciphering key would be                                   }
{                GZAYLJXKFWVUCEBTSRIHDQPONM                           }
{                                                                     }
{  Input:  A string of up to 80 letters entered by the user.          }
{                                                                     }
{  Output: Enciphering and deciphering keys in the file KEYS,         }
{          generated as described above.  These keys are also         }
{          displayed on the terminal.                                 }
{                                                                     }
{  This program contains non-standard Pascal statements to support    }
{  Turbo Pascal and Berkeley Pascal versions.  Non-standard Pascal    }
{  statements appear in the following locations:                      }
{    1. in procedures Initialize and CleanUp                          }
{  These non-standard statements are preceded by a comment that       }
{  begins with '## N'.                                                }
{---------------------------------------------------------------------}

type
  Key = array['a'..'z'] of char;      { encipherment or decipherment of the
                                        characters 'a' through 'z' }
  KeyString = array[1..80] of char;

var
  KEYS : text;                        { file to receive keys }
  EncipherKey : Key;                  { enciphering key }
  DecipherKey : Key;                  { deciphering key }
  InputString : KeyString;            { string entered by the user }
  StringLength : integer;             { length of InputString }

{*********************************************************************}

function Lowercase(Letter : char) : char;

  {-------------------------------------------------------------------}
  {  Lowercase is used to convert alphabetic characters to lower      }
  {  case.  If the input parameter is an upper case letter            }
  {  ('A' through 'Z'), the value of the function is the corresponding}
  {  lower case letter ('a' through 'z').  If the input parameter is  }
  {  any other character, the value of the function is equal to the   }
  {  input parameter.                                                 }
  {                                                                   }
  {  Input parameters:                                                }
  {    Letter - character to be converted to lower case               }
  {                                                                   }
  {  Output parameters : None                                         }
  {                                                                   }
  {  Function value : the value of the input parameter Letter,        }
  {                   converted to lower case if it was originally    }
  {                   in the range 'A' through 'Z'                    }
  {-------------------------------------------------------------------}

  var
    ConvertedLetter : char;       { the function value being computed }
    DistancePastA : integer;      { the distance in the alphabet between
                                    the letter being converted and 'A' }
  begin  { Lowercase }

    { if the input parameter is an upper case letter, convert it to
      the corresponding lower case letter.  otherwise, the converted
      letter is set equal to the input parameter. }

    if (Letter >= 'A') and (Letter <= 'Z') then
      begin
        DistancePastA := ord(Letter) - ord('A');
        ConvertedLetter := chr(ord('a') + DistancePastA)
      end  { if }
    else
      ConvertedLetter := Letter;

    { set the function value }

    Lowercase := ConvertedLetter

  end;  { Lowercase }

{*********************************************************************}

function Uppercase(Letter : char) : char;

  {-------------------------------------------------------------------}
  {  Uppercase is used to convert alphabetic characters to upper      }
  {  case.  If the input parameter is a lower case letter             }
  {  ('a' through 'z'), the value of the function is the corresponding}
  {  upper case letter ('A' through 'Z').  If the input parameter is  }
  {  any other character, the value of the function is equal to the   }
  {  input parameter.                                                 }
  {                                                                   }
  {  Input parameters:                                                }
  {    Letter - character to be converted to upper case               }
  {                                                                   }
  {  Output parameters : None                                         }
  {                                                                   }
  {  Function value : the value of the input parameter Letter,        }
  {                   converted to upper case if it was originally    }
  {                   in the range 'a' through 'z'                    }
  {-------------------------------------------------------------------}

  var
    ConvertedLetter : char;       { the function value being computed }
    DistancePastA : integer;      { the distance in the alphabet between
                                    the letter being converted and 'a' }
  begin  { Uppercase }

    { if the input parameter is a lower case letter, convert it to
      the corresponding upper case letter.  otherwise, the converted
      letter is set equal to the input parameter. }

    if (Letter >= 'a') and (Letter <= 'z') then
      begin
        DistancePastA := ord(Letter) - ord('a');
        ConvertedLetter := chr(ord('A') + DistancePastA)
      end  { if }
    else
      ConvertedLetter := Letter;

    { set the function value }

    Uppercase := ConvertedLetter

  end;  { Uppercase }

{*********************************************************************}

procedure Initialize;

  {-------------------------------------------------------------------}
  {  Initialize is called once at the beginning of the program.       }
  {  It opens the output file KEYS.                                   }
  {-------------------------------------------------------------------}

  begin  { Initialize }

    { ## Non-standard statements -- Turbo Pascal version }

    assign(KEYS,'KEYS');

    { End Turbo Pascal version }

    rewrite(KEYS)

  end;  { Initialize }

{*********************************************************************}

procedure GetString(var InputString : KeyString; var StringLength : integer);

  {-------------------------------------------------------------------}
  {  GetString is called to obtain the input string from the user.    }
  {  It prompts the user to enter a string of letters, then reads     }
  {  the string from the terminal (a maximum of 80 letters).          }
  {  Letters read are converted to lower case, and non-alphabetic     }
  {  characters are discarded.  If more than 80 letters are entered,  }
  {  the excess characters are skipped with no error message.         }
  {                                                                   }
  {  Input parameters: None                                           }
  {                                                                   }
  {  Output parameters:                                               }
  {    InputString - the string of characters read from the terminal, }
  {                  with letters converted to lower case and         }
  {                  non-alphabetic characters discarded.             }
  {    StringLength - the number of letters in InputString            }
  {-------------------------------------------------------------------}

  var
    CharacterCount : integer;    { number of characters placed in InputString }
    Letter : char;               { character read from terminal }
    ConvertedLetter : char;      { Letter converted to lower case }

  begin  { GetString }

    { prompt the user to enter a string of letters }

    writeln('Enter key string (1 to 80 characters): ');
    CharacterCount := 0;

    { read and process letters until end-of-line is reached, or until
      80 letters have been read }

    while (not eoln(input)) and (CharacterCount < 80) do
      begin

        { read a character and convert it to lower case }

        read(Letter);
        ConvertedLetter := Lowercase(Letter);

        { if the character read is alphabetic, add it to InputString }

        if (ConvertedLetter >= 'a') and (ConvertedLetter <= 'z') then
          begin
            CharacterCount := CharacterCount + 1;
            InputString[CharacterCount] := ConvertedLetter
          end  { if }
      end;  { while }

    { skip any unread input on the line }

    readln;

    { set the string length }

    StringLength := CharacterCount

  end;  { GetString }

{*********************************************************************}

procedure GenerateKey(InputString : KeyString; StringLength : integer;
                      var EncipherKey : Key);

  {-------------------------------------------------------------------}
  {  GenerateKey is called to generate the enciphering key from the   }
  {  input string entered by the user.  It uses an internal procedure }
  {  PutInKey to insert each letter from the input string into the    }
  {  key being constructed, unless that character is already present  }
  {  in the key.  Then it uses PutInKey to insert letters 'z'...'a'   }
  {  into the key (unless they are already present) to account for    }
  {  letters not in the input string.                                 }
  {                                                                   }
  {  Input parameters:                                                }
  {    InputString - the string of letters entered by the user        }
  {    StringLength - the length of InputString                       }
  {                                                                   }
  {  Output parameters:                                               }
  {    EncipherKey - the enciphering key generated from InputString   }
  {-------------------------------------------------------------------}

  var
    Letter : char;          { used as subscript and working variable }
    I : integer;            { used as subscript for InputString }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  procedure PutInKey(var EncipherKey : Key; Character : char);

  {-------------------------------------------------------------------}
  {  PutInKey is called with each character that is a candidate to be }
  {  placed into the enciphering key.  PutInKey scans the letters     }
  {  that are already in the key.  If the character to be inserted is }
  {  already present, it is ignored; otherwise, it is inserted at the }
  {  first unused position in the key.  It is assumed that unused     }
  {  positions in the key being constructed have been set to blanks.  }
  {                                                                   }
  {  Input parameters:                                                }
  {    EncipherKey - the partially constructed key.  Letters that are }
  {                  already in EncipherKey are assumed to appear at  }
  {                  the beginning of the key, followed by blanks.    }
  {    Character - the character that is a candidate to be inserted   }
  {                into EncipherKey                                   }
  {                                                                   }
  {  Output parameters:                                               }
  {    EncipherKey - the partially constructed key, with Character    }
  {                  added unless it was already present              }
  {-------------------------------------------------------------------}
    var
      Searching : boolean;      { searching for place to insert Character }
      Letter : char;            { used as subscript for EncipherKey }

    begin  { PutInKey }

      Searching := true;
      Letter := 'a';

      { scan EncipherKey until Character  or blank is found.  if a blank
        is found, replace it with Character }

      while Searching do
        begin
          if EncipherKey[Letter] = Character then

            { Character is already present -- stop searching }

            Searching := false

          else if EncipherKey[Letter] = ' ' then
            begin

              { a blank was found.  replace it with Character and
                stop searching }

              EncipherKey[Letter] := Character;
              Searching := false
            end  { if blank was found }

          else

            { neither Character or blank found yet -- go on to the
              next letter in EncipherKey }

            Letter := succ(Letter)

        end  { while }

    end;  { PutInKey }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  begin  { GenerateKey }

    { initialize EncipherKey to all blanks }

    for Letter := 'a' to 'z' do
      EncipherKey[Letter] := ' ';

    { insert letters from InputString into EncipherKey }

    for I := 1 to StringLength do
      PutInKey(EncipherKey,InputString[I]);

    { insert letters 'z', 'y',...,'a' not already in EncipherKey }

    for Letter := 'z' downto 'a' do
      PutInKey(EncipherKey,Letter)

  end;  { GenerateKey }

{*********************************************************************}

procedure ConvertKey(EncipherKey : Key; var DecipherKey : Key);

  {-------------------------------------------------------------------}
  {  ConvertKey generates the deciphering key that corresponds to a   }
  {  given enciphering key.  It operates by scanning the letters in   }
  {  the enciphering key.  For each letter, it makes the "inverse"    }
  {  entry in the deciphering key.  For example, if EncipherKey['a']  }
  {  is equal to 'j', then 'A' is entered in DecipherKey['j'].        }
  {                                                                   }
  {  Input parameters:                                                }
  {    EncipherKey - the enciphering key                              }
  {                                                                   }
  {  Output parameters:                                               }
  {    DecipherKey - the corresponding deciphering key                }
  {-------------------------------------------------------------------}

  var
    Letter : char;         { used as subscript for EncipherKey }
    CipherLetter : char;   { the character at position Letter in EncipherKey }

  begin  { ConvertKey }

    for Letter := 'a' to 'z' do
      begin

        { CipherLetter is the character at position Letter in EncipherKey }

        CipherLetter := EncipherKey[Letter];

        { enter Letter, converted to upper case, in position CipherLetter
          of DecipherKey }

        DecipherKey[CipherLetter] := Uppercase(Letter)
      end

  end;  { ConvertKey }

{*********************************************************************}

procedure WriteKeys(EncipherKey : Key; DecipherKey : Key);

  {-------------------------------------------------------------------}
  {  WriteKeys is called at the end of the program.  It writes the    }
  {  generated enciphering and deciphering keys to the file KEYS,     }
  {  and also displays these keys on the terminal.                    }
  {                                                                   }
  {  Input parameters:                                                }
  {    EncipherKey - the generated enciphering key                    }
  {    DecipherKey - the generated deciphering key                    }
  {                                                                   }
  {  Output parameters: None                                          }
  {-------------------------------------------------------------------}

  var
    Letter : char;      { used as subscript for EncipherKey and DecipherKey }

  begin  { WriteKeys }

    { write enciphering key to KEYS and to terminal }

    write('Enciphering key is  ');
    for Letter := 'a' to 'z' do
      begin
        write(EncipherKey[Letter]);
        write(KEYS,EncipherKey[Letter])
      end;
    writeln;
    writeln(KEYS);

    { write deciphering key to KEYS and to terminal }

    write('Deciphering key is  ');
    for Letter := 'a' to 'z' do
      begin
        write(DecipherKey[Letter]);
        write(KEYS,DecipherKey[Letter])
      end;
    writeln;
    writeln(KEYS)

  end;  { WriteKeys }

{*********************************************************************}

procedure CleanUp;

  {-------------------------------------------------------------------}
  {  CleanUp is called once at the end of the program.  It closes     }
  {  the file KEYS, if this is required in the Pascal version being   }
  {  used, and waits for the user to enter a carriage return before   }
  {  exiting the program.                                             }
  {-------------------------------------------------------------------}

  begin  { CleanUp }

    { ## Non-standard statements -- Turbo Pascal version }

    close(KEYS);

    { End Turbo Pascal version }

    write('Press return to exit program');
    readln

  end;  { CleanUp}

{*********************************************************************}

begin  { main program }

  Initialize;

  { get input string from terminal }

  GetString(InputString,StringLength);

  { generate enciphering and deciphering keys }

  GenerateKey(InputString,StringLength,EncipherKey);
  ConvertKey(EncipherKey,DecipherKey);

  { write keys to file and to terminal }

  WriteKeys(EncipherKey,DecipherKey);
  CleanUp

end.
