program Letters1(input,output,TEXTDATA);

{---------------------------------------------------------------------}
{  This program reads an input file and counts the number of          }
{  times each letter of the alphabet occurs in this file.  It writes  }
{  an alphabetical listing that shows the total number of occurrences }
{  of each letter, the relative frequency of each letter, and the     }
{  total number of occurrences of all letters.                        }
{                                                                     }
{  Capitalization of letters is not considered significant -- for     }
{  example, 'A' and 'a' are considered to be the same letter for the  }
{  purposes of letter count and frequency calculations.  Characters   }
{  in the input file that are not alphabetic are ignored.             }
{                                                                     }
{  Terminal screen control routines are used to clear the screen      }
{  initially, and to position the cursor so that the output listing   }
{  appears in two columns.                                            }
{                                                                     }
{  Input :   File TEXTDATA, containing the characters which are to    }
{            be counted.                                              }
{                                                                     }
{  Output :  An alphabetical listing showing the total number of      }
{            occurrences, and the relative frequency of occurrence,   }
{            of each letter of the alphabet in the input file.        }
{                                                                     }
{  This program contains non-standard Pascal statements to support    }
{  Turbo Pascal and Berkeley Pascal versions.  Non-standard Pascal    }
{  statements appear in the following locations:                      }
{    1. in the 'uses' clause immediately following this comment block }
{    2. in the screen control procedures ClearScreen and MoveCursor   }
{    3. in procedures Initialize and CleanUp                          }
{  These non-standard statements are preceded by a comment that       }
{  begins with '## N'.                                                }
{---------------------------------------------------------------------}

  { ## Non-standard statements -- Turbo Pascal version }

  uses Crt;

  { End Turbo Pascal version }


type
  CountOfLetters = array ['a'..'z'] of integer;

var
  TEXTDATA : text;                    { input file }
  LetterCounts : CountOfLetters;      { count of occurrences of each letter }
  Character : char;                   { used as subscript for LetterCounts }
  TotalLetters : integer;             { total number of letters counted }

{*********************************************************************}

function Lowercase(Letter : char) : char;

  {-------------------------------------------------------------------}
  {  Lowercase is used to convert alphabetic characters to lower      }
  {  case.  If the input parameter is an upper case letter            }
  {  ('A' through 'Z'), the value of the function is the corresponding}
  {  lower case letter ('a' through 'z').  If the input parameter is  }
  {  any other character, the value of the function is equal to the   }
  {  input parameter.                                                 }
  {                                                                   }
  {  Input parameters:                                                }
  {    Letter - character to be converted to lower case               }
  {                                                                   }
  {  Output parameters : None                                         }
  {                                                                   }
  {  Function value : the value of the input parameter Letter,        }
  {                   converted to lower case if it was originally    }
  {                   in the range 'A' through 'Z'                    }
  {-------------------------------------------------------------------}

  var
    ConvertedLetter : char;       { the function value being computed }
    DistancePastA : integer;      { the distance in the alphabet between
                                    the letter being converted and 'A' }
  begin  { Lowercase }

    { if the input parameter is an upper case letter, convert it to
      the corresponding lower case letter.  otherwise, the converted
      letter is set equal to the input parameter. }

    if (Letter >= 'A') and (Letter <= 'Z') then
      begin
        DistancePastA := ord(Letter) - ord('A');
        ConvertedLetter := chr(ord('a') + DistancePastA)
      end  { if }
    else
      ConvertedLetter := Letter;

    { set the function value }

    Lowercase := ConvertedLetter

  end;  { Lowercase }

{*********************************************************************}

{ ## Non-standard statements -- Berkeley Pascal version }
{
procedure ClrScr; external;
}
{ End Berkeley Pascal version }

procedure ClearScreen;

  {-------------------------------------------------------------------}
  {  ClearScreen clears the terminal screen and positions the         }
  {  cursor at the upper left corner of the screen.  It calls a       }
  {  non-standard Pascal library routine to perform this operation.   }
  {-------------------------------------------------------------------}

  begin

    ClrScr   { call library routine ClrScr to clear screen }

  end;  { ClearScreen }

{*********************************************************************}

{ ## Non-standard statements -- Berkeley Pascal version }
{
procedure GotoXY(X, Y : integer); external;
}
{ End Berkeley Pascal version }

procedure MoveCursor(Row, Col : integer);

  {-------------------------------------------------------------------}
  {  MoveCursor moves the cursor on the terminal screen to the        }
  {  row and column given by its input parameters.  The next write    }
  {  or writeln operation that sends output to the terminal will      }
  {  begin writing at the row and column location designated.  The    }
  {  procedure calls a non-standard Pascal library routine to perform }
  {  this operation.                                                  }
  {                                                                   }
  {  Input parameters:                                                }
  {    Row,Col - the row (line) and column (horizontal position) on   }
  {          the screen to which the cursor is to be moved.  The      }
  {          upper left-hand corner of the screen is row 1, column 1. }
  {-------------------------------------------------------------------}

  begin

    { call library routine GotoXY to move the cursor -- note that
      GotoXY expects the column value as its first parameter }

    GotoXY(Col,Row)

  end;  { MoveCursor }

{*********************************************************************}

procedure Initialize(var LetterCounts : CountOfLetters;
                     var TotalLetters : integer);

  {-------------------------------------------------------------------}
  {  Initialize is called once at the beginning of the program.  It   }
  {  opens the input file TEXTDATA and initializes the total number   }
  {  of letters and the array of letter counts to zero.               }
  {                                                                   }
  {  Input parameters : None                                          }
  {                                                                   }
  {  Output parameters:                                               }
  {    LetterCounts - initialized to all zeros                        }
  {    TotalLetters - initialized to zero                             }
  {-------------------------------------------------------------------}

  var
    Character : char;          { used as subscript for LetterCounts }

  begin  { initialize }

    { ## Non-standard statements -- Turbo Pascal version }

      assign(TEXTDATA,'TEXTDATA');

    { End Turbo Pascal version }

    { open data file }

    reset(TEXTDATA);

    { initialize total letters and all letter counts to all zero }

    TotalLetters := 0;
    for Character := 'a' to 'z' do
      LetterCounts[Character] := 0

  end;  { Initialize }

{*********************************************************************}

procedure CountCharacters(var LetterCounts : CountOfLetters;
                          var TotalLetters : integer);

  {-------------------------------------------------------------------}
  {  CountCharacters is called once.  It reads TEXTDATA, one character}
  {  at a time, until end-of-file.  For each alphabetic character     }
  {  read, it adds one to the corresponding count in LetterCounts.    }
  {  After all letters have been counted, the total number of letters }
  {  is calculated and stored in TotalLetters.                        }
  {                                                                   }
  {  Input parameters:                                                }
  {    LetterCounts - all counts assumed to be initialized to zero    }
  {    TotalLetters - assumed to be initialized to zero               }
  {                                                                   }
  {  Output parameters:                                               }
  {    LetterCounts - counts of each alphabetic character in TEXTDATA }
  {    TotalLetters - the total number of letters counted             }
  {-------------------------------------------------------------------}

  var
    Character : char;     { character read from TEXTDATA }
    Letter : char;        { used as subscript for LetterCounts }

  begin  { CountCharacters }

    { read TEXTDATA, one character at a time, until end-of-file }

    while not eof(TEXTDATA) do
      begin
        if eoln(TEXTDATA) then

          { if the end of a line has been reached, go to the next line }

          readln(TEXTDATA)
        else
          begin

            { if end of line has not been reached, read the next
              character and convert it to lower case }

            read(TEXTDATA,Character);
            Character := Lowercase(Character);

            { if the converted character is 'a' through 'z', add one
              to the corresponding count in LetterCounts }

            if (Character >= 'a') and (Character <= 'z') then
              LetterCounts[Character] := LetterCounts[Character] + 1
          end  { else }
      end;  { while }

    { calculate the total number of letters counted }

    for Letter := 'a' to 'z' do
      TotalLetters := TotalLetters + LetterCounts[Letter]

  end;  { CountCharacters }

{*********************************************************************}

procedure WriteResults(LetterCounts : CountOfLetters; TotalLetters : integer);

  {-------------------------------------------------------------------}
  {  WriteResults is called once at the end of the program.  It       }
  {  displays a table of letters, in alphabetical order, with the     }
  {  number of occurrences and relative frequency of each letter.     }
  {  Screen control routines are called to clear the screen and to    }
  {  position the cursor to display the table in two columns.         }
  {                                                                   }
  {  Input parameters:                                                }
  {    LetterCounts - count of number of occurrences of each letter   }
  {    TotalLetters - total number of letters counted                 }
  {                                                                   }
  {  Output parameters: None                                          }
  {-------------------------------------------------------------------}

  var
    Character : char;          { used as subscript for LetterCounts }
    LineNumber : integer;      { count of number of lines written }
    Frequency : real;          { relative frequency of current letter }

  begin  { WriteResults }

    { clear terminal screen and write headings }

    ClearScreen;
    writeln('Letter   Count    Frequency         Letter   Count    Frequency');

    { first line to be written is line number 1 }

    LineNumber := 1;

    { display letter counts and frequencies }

    for Character := 'a' to 'z' do
      begin

        { position cursor in the left-hand column for lines 1-13, or in
          the right-hand column for lines 14-26 }

        if LineNumber <= 13 then
          MoveCursor(LineNumber+2,4)
        else
          MoveCursor(LineNumber-11,40);

        { write letter, count, and relative frequency }

        write(Character,LetterCounts[Character]:10);
        Frequency := LetterCounts[Character] / TotalLetters;
        writeln(Frequency:12:2);

        { update line number }

        LineNumber := LineNumber + 1
      end;  { for }

    { move cursor to bottom of table and write total letter count }

    MoveCursor(17,1);
    writeln('Total letters = ',TotalLetters)

  end;  { WriteResults }

{*********************************************************************}

procedure CleanUp;

  {-------------------------------------------------------------------}
  {  CleanUp is called once, at the end of the program.  It closes    }
  {  the input file, if this is required in the Pascal version being  }
  {  used, and waits for the user to enter a carriage return before   }
  {  exiting the program.                                             }
  {-------------------------------------------------------------------}

  begin

    { ## Non-standard statements -- Turbo Pascal version }

    close(TEXTDATA);

    { End Turbo Pascal version }

    write('Press return to exit program');
    readln

  end;

{*********************************************************************}

begin  { main program }

  Initialize(LetterCounts,TotalLetters);

  { read input file and count alphabetic characters }

  CountCharacters(LetterCounts,TotalLetters);

  { display table of results }

  WriteResults(LetterCounts,TotalLetters);
  CleanUp

end.
